/*-
 * Copyright (C) @BABOLO  2005 Oct 14
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: iso8583.h,v 1.4 2005/11/12 21:12:28 babolo Exp $
 */

/* flags */
#define ISO8583_MSZ    0x00FFFF    /* MAX size of ISO8583 message     */
#define ISO8583_FMS    0x010000    /* Package has own message to free */

/* Representations */
#define ISO8583_B_LITERAL     0
#define ISO8583_I_CHAR        1
#define ISO8583_B_LLCHAR      2
#define ISO8583_B_LLLCHAR     3
#define ISO8583_B_NUMERIC     4
#define ISO8583_B_NUMERICl    5
#define ISO8583_B_LLNUM       6
#define ISO8583_B_LLNUMl      7
#define ISO8583_B_TIME        8
#define ISO8583_B_DATE        9
#define ISO8583_B_DATET      10
#define ISO8583_B_BINARY     11
#define ISO8583_B_LLBINARY   12
#define ISO8583_B_LLLBINARY  13
#define ISO8583_E_MAX        14

/* Restriction */
#define ISO8583_PACK_BOUND   65

/*    ISO8583_Test() */
#define ISO8583_PRES 1
#define ISO8583_VALU 2
#define ISO8583_REFU 4
#define ISO8583_ISVL (ISO8583_PRES | ISO8583_VALU)

typedef struct {
    u_int32_t flag;
    u_int32_t pad0;
    ssize_t   bound;
    ssize_t   pad1;
    u_int64_t premask;
    u_int64_t pad64;
    struct {
        int16_t  convertor;
        int16_t  len;
        char    *description;
    }         fld[ISO8583_PACK_BOUND];
} ISO8583_Package;

typedef struct {
    u_int32_t              flags;
    u_int32_t              totlen;
    time_t                 now;
    char                  *tz;
    size_t                 msz;
    const ISO8583_Package *package;
    u_char                *refo[ISO8583_PACK_BOUND];
} ISO8583_Pack;

const ISO8583_Package *ISO8583_Take __P((char *nm));
int           ISO8583_List __P((int first, int len, FILE *prt));
ISO8583_Pack *ISO8583_Mark __P(( u_int32_t flags
                               , const ISO8583_Package *package
                               , u_char *in
                               , size_t msz
                               , char *tzname
                              ));
int           ISO8583_Test __P((ISO8583_Pack *pack, int fno));
u_int64_t     ISO8583_Val  __P((ISO8583_Pack *pack, int fno));
char         *ISO8583_Ref  __P((ISO8583_Pack *pack, int fno));
int           ISO8583_Asn  __P((ISO8583_Pack *pack, int fno, u_int64_t val));
int           ISO8583_Cpy  __P((ISO8583_Pack *pack, int fno, u_char *vly));
void          ISO8583_Save __P((ISO8583_Pack *pack));
void          ISO8583_Free __P((ISO8583_Pack *pack));
