/*-
 * Copyright (C) @BABOLO  2005 Nov 2
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: val.cc,v 1.3 2005/11/12 21:12:28 babolo Exp $
 */

/***************************************************/
/*     codex.val()    */
/***************************************************/

static u_int64_t
lv(int mode, int len, u_char *ref) {
    u_int64_t val;
    int i, a;

    for (val = 0, i = (~mode & len & 1); i < len + (~mode & len & 1); i++) {
        a = ((i & 1) ? *ref++ : (*ref >> 4)) & 0x0f;
        if  (a > 9) return(-MS_ILL);
        val = val * 10 + a;
    }
    return(val);
}

static int
expnd(ISO8583_Pack *pack, int fno, int mode) {
#define TIMEPARTS 7
    int i;
    char *ref;
    struct tm tn, tm;
    int len, cn, cm, *an[TIMEPARTS], *am[TIMEPARTS];
    int lo[TIMEPARTS] = { 0,  0,  1,  1,  0,  0,  0};
    int hi[TIMEPARTS] = {20, 99, 12, 31, 23, 59, 60};

    len = pack->package->fld[fno].len;
    ref = pack->refo[fno];
    if  (len & 1 || mode & 1) {
        ifBLIN_QV2(pack->flags) warnx("expnd: illegal parameters");
        errno = EINVAL;
        return(-1);

    }
    len >>= 1;
    mode >>= 1;
    bzero(&tm, sizeof(struct tm));
    bzero(&tn, sizeof(struct tm));
    gmtime_r(&(pack->now), &tn);
    cn = tn.tm_year / 100 + 19;
    tn.tm_year = tn.tm_year % 100;
    tn.tm_mon++;
    an[0] = &cn;            am[0] = &cm;
    an[1] = &(tn.tm_year); am[1] = &(tm.tm_year);
    an[2] = &(tn.tm_mon);  am[2] = &(tm.tm_mon);
    an[3] = &(tn.tm_mday); am[3] = &(tm.tm_mday);
    an[4] = &(tn.tm_hour); am[4] = &(tm.tm_hour);
    an[5] = &(tn.tm_min);  am[5] = &(tm.tm_min);
    an[6] = &(tn.tm_sec);  am[6] = &(tm.tm_sec);
    for (i = 0; i < TIMEPARTS; i++) {
        if  (i < mode) {
            *(am[i]) = *(an[i]);
            ifBLIN_QV5(pack->flags) fprintf(stderr, "expnd< %d %02d\n", i, *(am[i]));
        } else if (i == mode) {
            *(am[i]) = lv(0, 2, ref++);
            *(am[i - 1]) += kolo(hi[i] - lo[i] + 1, *(an[i]), *(am[i]));
            ifBLIN_QV5(pack->flags) fprintf(stderr, "expnd= %d %02d%02d\n", i, *(am[i - 1]), *(am[i]));
        } else if (i < len + mode) {
            *(am[i]) = lv(0, 2, ref++);
            ifBLIN_QV5(pack->flags) fprintf(stderr, "expnd> %d %02d\n", i, *(am[i]));
        } else {
            *(am[i]) = lo[i];
            ifBLIN_QV5(pack->flags) fprintf(stderr, "expnd  %d %02d\n", i, *(am[i]));
    }   }
    tm.tm_mon--;
    tm.tm_year = (tm.tm_year + cm * 100) - 1900;
    return((u_int64_t)timegm(&tm));
}

static u_int64_t
ltri(int len, u_char *ref) {
    u_int64_t val;
    int i;

    for (val = 0, i = 0; i < len; i++) {
        val <<= 8;
        val |= *ref++;
    }
    return(val);
}

static u_int64_t
ltrl(ISO8583_Pack *pack, int fno) {
    return(ltri(pack->package->fld[fno].len, pack->refo[fno]));
}

static u_int64_t
lvl(ISO8583_Pack *pack, int fno) {
    return(lv(1, pack->package->fld[fno].len, pack->refo[fno]));
}

static u_int64_t
lvr(ISO8583_Pack *pack, int fno) {
    return(lv(0, pack->package->fld[fno].len, pack->refo[fno]));
}

static u_int64_t
lvnl(ISO8583_Pack *pack, int fno) {
    u_char *ref = pack->refo[fno];
    return(lv(1, lv(0, 2, ref), ref + 1));
}

static u_int64_t
lvnr(ISO8583_Pack *pack, int fno) {
    u_char *ref = pack->refo[fno];
    return(lv(0, lv(0, 2, ref), ref + 1));
}

static u_int64_t
tim(ISO8583_Pack *pack, int fno) {
    return(expnd(pack, fno, 14 - pack->package->fld[fno].len));
}

static u_int64_t
dat(ISO8583_Pack *pack, int fno) {
    return(expnd(pack, fno, 2));
}

static u_int64_t
datt(ISO8583_Pack *pack, int fno) {
    return(expnd(pack, fno, 8 - pack->package->fld[fno].len));
}
